@@ -19,6 +19,8 @@ from astropy.visualization import PercentileInterval
 from astroquery.skyview import SkyView
 from astropy.utils.data import clear_download_cache
 
+from utils import write_products, write_bytesio_to_file, write_zipped_fits_file
+
 Run, Instance, Detection, Product, Source  = None, None, None, None, None
 SourceDetection, Comment, Tag, TagDetection, TagSourceDetection = None, None, None, None, None
 
@@ -83,9 +85,12 @@ def get_catalog(tag):
     
     # Add columns to the table
     for field in source_field_names:
-        table[field] = [getattr(s, field) for s in sources]
+        if field == 'name':
+            table[field] = [getattr(s, field) for s in sources]
+        else:
+            table[field] = np.array([getattr(s, field) for s in sources], dtype=float)
     for field in detection_field_names:
-        table[field] = [getattr(d, field) for d in detections]
+        table[field] = np.array([getattr(d, field) for d in detections], dtype=float)
     
     # Extract and add comments, if any
     column_comments = []
@@ -99,7 +104,7 @@ def get_catalog(tag):
     # Extract and add tags, if any
     column_tags = []
     for i in range(len(table)):
-        column_tags.append([]);
+        column_tags.append([])
         tags = TagSourceDetection.objects.filter(source_detection_id=SourceDetection.objects.get(detection_id=table["id"][i]))
         for tag in tags:
             column_tags[i].append(Tag.objects.get(id=tag.tag_id).name)
@@ -108,6 +113,62 @@ def get_catalog(tag):
     return table
 
 
+def save_catalog(tag, *args, **kwargs):
+    """Write catalog of tagged sources. Remove object columns for write to file.
+
+    """
+    table = get_catalog(tag)
+    table.remove_columns(['comments', 'tags'])
+    table.write(*args, **kwargs)
+
+
+def save_products_for_source(tag, source_name, *args, **kwargs):
+    """Save source finding output products for a given source name.
+
+    """
+    table = get_catalog(tag)
+    try:
+        idx = list(table['name']).index(source_name)
+        row = table[idx]
+    except Exception as e:
+        sys.stderr.write("Could not find source with provided name in tagged data.")
+        return None
+    detection = Detection.objects.get(id=row['id'])
+    products = Product.objects.get(detection=detection)
+
+    name = source_name.replace(' ', '_')
+    parent = f'{name}_products'
+    if not os.path.isdir(parent):
+        os.mkdir(parent)
+    
+    # Write fits files
+    write_products(products, f'{parent}/{name}')
+    
+    return
+
+
+def save_products(tag, *args, **kwargs):
+    """Save source finding output products for a given tag
+
+    """
+    table = get_catalog(tag)
+    parent = '%s_products' % tag.replace(' ', '_')
+    if not os.path.isdir(parent):
+        os.mkdir(parent)
+
+    for row in table:
+        name = row['name'].replace(' ', '_')
+        if not os.path.isdir(f'{parent}/{name}'):
+            os.mkdir(f'{parent}/{name}')
+        detection = Detection.objects.get(id=row['id'])
+        products = Product.objects.get(detection=detection)
+        write_products(products, f'{parent}/{name}/{name}')
+
+    os.system(f'tar -czf {parent}.tar.gz {parent}')
+
+    return
+
+
 # Print list of supported tags
 def print_tags():
     tags = Tag.objects.all()
@@ -141,9 +202,10 @@ def retrieve_dss_image(longitude, latitude, width, height):
         coordinates="J2000",
         projection="Tan",
         width=width*u.deg,
-        height=height*u.deg
+        height=height*u.deg,
+        cache=None
     )
-    return hdulist[0][0];
+    return hdulist[0][0]
 
 
 # Create overview plot
@@ -222,7 +284,7 @@ def overview_plot(id):
     xmin = np.nanmin(xaxis)
     xmax = np.nanmax(xaxis)
     ymin = np.nanmin(data)
-    ymax = np.nanmax(data);
+    ymax = np.nanmax(data)
     ymin -= 0.1 * (ymax - ymin)
     ymax += 0.1 * (ymax - ymin)
     ax4 = plt.subplot(2, 2, 4)
@@ -237,8 +299,24 @@ def overview_plot(id):
     fig.canvas.draw()
     plt.tight_layout()
     
-    # Clean up
-    clear_download_cache(pkgname="astroquery")
-    clear_download_cache()
-    
     return plt
+
+
+def save_overview(tag, *args, **kwargs):
+    """Save overview plots for tagged sources
+
+    """
+    table = get_catalog(tag)
+    parent = '%s_overview' % tag.replace(' ', '_')
+    if not os.path.isdir(parent):
+        os.mkdir(parent)    
+
+    for row in table:
+        name = row['name'].replace(' ', '_')
+        p = overview_plot(row['id'])
+        p.savefig(f"{parent}/{name}_overview.png")
+        p.close()
+
+    os.system(f'tar -czf {parent}.tar.gz {parent}')
+
+    return
